#!perl -w

use strict;
use Test::More tests => 5;

use_ok('Net::Google::Calendar');
use_ok('Net::Google::Calendar::Entry');
use_ok('Net::Google::Calendar::Person');
use_ok('Net::Google::Calendar::Calendar');
use_ok('Net::Google::Calendar::WebContent');

